module DoctorStrange
  class HealthController < ActionController::Base
    protect_from_forgery with: :exception

    if Rails.version.starts_with? '3'
      before_filter :authenticate_with_basic_auth
    else
      before_action :authenticate_with_basic_auth
    end

    def check
      @statuses = statuses
      @app_name = DoctorStrange.configuration.app_name

      respond_to do |format|
        format.html
        format.json do
          render json: statuses.to_json
        end
        format.xml do
          render xml: statuses.to_xml
        end
      end
    end

    private

      def statuses
        res = DoctorStrange.check(request: request)
        res.merge(env_vars)
      end

      def env_vars
        v = DoctorStrange.configuration.environment_variables || {}
        v.empty? ? {} : { environment_variables: v }
      end

      def authenticate_with_basic_auth
        return true unless DoctorStrange.configuration.basic_auth_credentials

        credentials = DoctorStrange.configuration.basic_auth_credentials
        authenticate_or_request_with_http_basic do |name, password|
          name == credentials[:username] && password == credentials[:password]
        end
      end
  end
end
